#!/usr/bin/perl -w
# -----------------------------------------------------------------------------
# Copyright (c) 2007 Morovia Corporation. All rights reserved.
# 
# code39gen.pl - a perl utility to create code 39 barcode strings.
#
# This utility is part of Morovia Code39 Scalable Font Pack which is licensed
# as a single product, and its component parts may not be distributed,
# transferred, or used separately.
# You must not remove this notice, or any other, from this software.
# ----------------------------------------------------------------------------

# ==================
# Globals
# ==================
my $bChecksum=0;                 # if checksum is required
my $bExtendedMode=0;             # if processing extended mode
my $message="";                  # data to be encoded

my $version="1.0";               # program version
my $bShowLogo=0;                 # if we display logo

# =====================
# forward subroutine declarations
# =====================
sub usage();
sub Code39($);
sub Code39Mod43($);
sub Code39Ascii($ $);

my %code39value_table = (
	"0"=>0, "1"=>1, "2"=>2, "3"=>3, "4"=>4, 
	"5"=>5, "6"=>6, "7"=>7, "8"=>8, "9"=>9,
	"A"=>10, "B"=>11, "C"=>12, "D"=>13, "E"=>14,
	"F"=>15, "G"=>16, "H"=>17, "I"=>18, "J"=>19,
	"K"=>20, "L"=>21, "M"=>22, "N"=>23, "O"=>24,
	"P"=>25, "Q"=>26, "R"=>27, "S"=>28, "T"=>29,
	"U"=>30, "V"=>31, "W"=>32, "X"=>33, "Y"=>34,
	"Z"=>35, "-"=>36, "."=>37, " "=>38, "\$"=>39,
	"/"=>40, "+"=>41, "%"=>42,
);

my @code39AsciiChart = (
'%U', 
'$A', '$B', '$C', '$D', '$E', '$F', '$G', '$H', '$I', '$J', '$K', '$L', '$M', 
'$N', '$O', 
'$P', '$Q', '$R', '$S', '$T', '$U', '$V', '$W', '$X', '$Y', '$Z', 
'%A', '%B', '%C', '%D', '%E', 
' ',
'/A', '/B', '/C', '/D', '/E', '/F', '/G', '/H', '/I', '/J', '/K', '/L', '-', 
'.', '/O',
'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '/Z', '%F', '%G', '%H', '%I', 
'%J','%V',
'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 
'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '%K', '%L', '%M', '%N', 
'%O', ,'%W', 
'+A', '+B', '+C', 
'+D', '+E', '+F', '+G', '+H', '+I', '+J', '+K', '+L', '+M', '+N', '+O', '+P', 
'+Q', '+R', 
'+S', '+T', '+U', '+V', '+W', '+X', '+Y', '+Z', '%P', '%Q', '%R', '%S', ,'%T'
);

my $code39set = join("", keys(%code39value_table) );

for(my $i=0; $i<@ARGV; $i++)
{
	$arg = $ARGV[$i];
	if ( substr($arg, 0, 1) eq "-" || substr($arg, 0, 1) eq "/" ) {
		$parameter = substr($arg, 1);
		if ( "h" eq $parameter ) {
			usage();
			exit 0;
		}
		elsif ( "k" eq $parameter ) {
			$bChecksum=1;
		}
		elsif ( "e" eq $parameter ) {
			$bExtendedMode=1;
		}
		else {
			print(STDERR "Error: invalid option $arg. Type -h for help.\n");
			exit 1;
		}
	}
	else {  # data 
		if ( length($message) > 0 ) {
			print(STDERR "Error: multiple data arguments were found. Type -h for help.\n");
			exit 2;
		}
		$message = $arg;
	}
}

if ( length($message)==0 ) {
	print(STDERR "Error: no data is specified.\n");
	exit 3;
}

if ($bExtendedMode) {
	print(STDOUT Code39Ascii($message, $bChecksum));
} 
elsif ( $bChecksum) {
	print(STDOUT Code39Mod43($message) );
}
else {
	print STDOUT Code39($message);
}


# ========================
# Display usage.
# ========================
sub usage() {
    print STDERR << "EOF";

Morovia (R) code39gen utility version $version
Copyright (c) Morovia Corporation 2007. All rights reserved.
    
usage: $0 [-h] [-k] [-e] [data]

 -h        : this (help) message
 -k        : add checksum 
 -e        : extended code 39 
 data      : data to be encoded

example: $0 -k PN199810

EOF
  exit;
}


# ==========================================
# returns barcode string for normal code 39
# ==========================================
sub Code39($) {
	my $arg = shift(@_);
	
	for(my $index=0; $index<length($arg); $index++) {
		my $ch = substr($arg, $index, 1);
		if ( index($code39set, $ch) < 0 ) {
			print STDERR "Error: invalid character $ch at index $index.\n";
			exit 4;
		}
	}
	
	$arg =~ s/ /=/g;
	return "*" . $arg . "*";
}

#==========================================
# return barcode string with check digit appended
#==========================================
sub Code39Mod43($) {
	my $arg = shift(@_);
	
	my $total_sum = 0;
	for(my $index=0; $index<length($arg); $index++) {
		my $ch = substr($arg, $index, 1);
		if ( index($code39set, $ch) < 0 ) {
			print STDERR "Error: invalid character $ch at index $index.\n";
			exit 4;
		}
		$total_sum += $code39value_table{$ch};
		if ($total_sum > 43 ) { $total_sum -= 43; }
	}
	my $checkdigit_value = $total_sum % 43;
	
	#lookup the corresponding code39 character
	my %reverse_code39table  = reverse %code39value_table;
	$arg = $arg . $reverse_code39table{$checkdigit_value};
	
	return Code39($arg);
}

#=================================================
# Code39FullASCII encodes 0-127 ASCII values
#=================================================
sub Code39Ascii($ $) {
	my ($arg, $bCheckDigit) = @_;
	my $converted = "";
	
	for(my $index=0; $index<length($arg); $index++) {
		my $ch = substr($arg, $index, 1);

		if ( ord($ch) < 0 || ord($ch) > 127 ) {
			print STDERR "Error: invalid character $ch at index $index.\n";
			exit 4;
		}
		$converted .= $code39AsciiChart[ord($ch)];
	}
	
	if ( $bCheckDigit ) {
		return Code39Mod43($converted);
	}
	else {
		return Code39($converted);
	}
}

